@@ -6,17 +6,17 @@
 
 #include "folder_selector.h"
 #include <zen/thread.h>
-#include <zen/file_access.h>
-#include <wx/dirdlg.h>
+#include <zen/process_exec.h>
 #include <wx/scrolwin.h>
+#include <wx+/bitmap_button.h>
 #include <wx+/popup_dlg.h>
-#include <wx+/context_menu.h>
 #include <wx+/image_resources.h>
 #include "small_dlgs.h" //includes structures.h, which defines "AFS"
 #include "../afs/concrete.h"
 #include "../afs/native.h"
-#include "../icon_buffer.h"
+#include "../afs/gdrive.h"
 
+    #include <wx/dirdlg.h>
 
 
 using namespace zen;
@@ -41,8 +41,17 @@ void setFolderPathPhrase(const Zstring& folderPathPhrase, FolderHistoryBox* comb
     else
         tooltipWnd.SetToolTip(utfTo<wxString>(folderPathPhraseFmt));
 
+    auto trimTrailingSep = [](Zstring path)
+    {
+        if (endsWith(path, Zstr('/')) ||
+            endsWith(path, Zstr('\\')))
+            path.pop_back();
+        return path;
+    };
+
     if (staticText) //change static box label only if there is a real difference to what is shown in wxTextCtrl anyway
-        staticText->SetLabel(equalNoCase(appendSeparator(trimCpy(folderPathPhrase)), appendSeparator(folderPathPhraseFmt)) ?
+        staticText->SetLabel(equalNoCase(trimTrailingSep(trimCpy(folderPathPhrase)),
+                                         trimTrailingSep(folderPathPhraseFmt)) ?
                              wxString(_("Drag && drop")) : utfTo<wxString>(folderPathPhraseFmt));
 }
 
@@ -53,8 +62,7 @@ void setFolderPathPhrase(const Zstring& folderPathPhrase, FolderHistoryBox* comb
 
 namespace fff
 {
-wxDEFINE_EVENT(EVENT_ON_FOLDER_SELECTED,    wxCommandEvent);
-wxDEFINE_EVENT(EVENT_ON_FOLDER_MANUAL_EDIT, wxCommandEvent);
+wxDEFINE_EVENT(EVENT_ON_FOLDER_SELECTED, wxCommandEvent);
 }
 
 
@@ -94,12 +102,13 @@ FolderSelector::FolderSelector(wxWindow*         parent,
     if (dropWindow2_)
         setupDragDrop(*dropWindow2_);
 
-    selectAltFolderButton_.SetBitmapLabel(loadImage("cloud_small"));
+    setImage(selectAltFolderButton_, loadImage("cloud_small"));
 
     //keep folderSelector and dirpath synchronous
     folderComboBox_       .Bind(wxEVT_MOUSEWHEEL,                &FolderSelector::onMouseWheel,          this);
     folderComboBox_       .Bind(wxEVT_COMMAND_TEXT_UPDATED,      &FolderSelector::onEditFolderPath,      this);
-    folderComboBox_       .Bind(wxEVT_COMMAND_COMBOBOX_SELECTED, &FolderSelector::onHistoryPathSelected, this);
+    //folderComboBox_.Bind(wxEVT_COMMAND_COMBOBOX_SELECTED, &FolderSelector::onHistoryPathSelected, this);
+    // => wxEVT_COMMAND_COMBOBOX_SELECTED implies wxEVT_COMMAND_TEXT_UPDATED
     selectFolderButton_   .Bind(wxEVT_COMMAND_BUTTON_CLICKED,    &FolderSelector::onSelectFolder,        this);
     selectAltFolderButton_.Bind(wxEVT_COMMAND_BUTTON_CLICKED,    &FolderSelector::onSelectAltFolder,     this);
 }
@@ -114,10 +123,11 @@ FolderSelector::~FolderSelector()
 
     [[maybe_unused]] bool ubOk3 = folderComboBox_       .Unbind(wxEVT_MOUSEWHEEL,                &FolderSelector::onMouseWheel,          this);
     [[maybe_unused]] bool ubOk4 = folderComboBox_       .Unbind(wxEVT_COMMAND_TEXT_UPDATED,      &FolderSelector::onEditFolderPath,      this);
-    [[maybe_unused]] bool ubOk5 = folderComboBox_       .Unbind(wxEVT_COMMAND_COMBOBOX_SELECTED, &FolderSelector::onHistoryPathSelected, this);
+    //[[maybe_unused]] bool ubOk5 = folderComboBox_ .Unbind(wxEVT_COMMAND_COMBOBOX_SELECTED, &FolderSelector::onHistoryPathSelected, this);
+    // => wxEVT_COMMAND_COMBOBOX_SELECTED implies wxEVT_COMMAND_TEXT_UPDATED
     [[maybe_unused]] bool ubOk6 = selectFolderButton_   .Unbind(wxEVT_COMMAND_BUTTON_CLICKED,    &FolderSelector::onSelectFolder,        this);
     [[maybe_unused]] bool ubOk7 = selectAltFolderButton_.Unbind(wxEVT_COMMAND_BUTTON_CLICKED,    &FolderSelector::onSelectAltFolder,     this);
-    assert(ubOk1 && ubOk2 && ubOk3 && ubOk4 && ubOk5 && ubOk6 && ubOk7);
+    assert(ubOk1 && ubOk2 && ubOk3 && ubOk4 && /*ubOk5 &&*/ ubOk6 && ubOk7);
 }
 
 
@@ -174,21 +184,11 @@ void FolderSelector::onItemPathDropped(FileDropEvent& event)
 }
 
 
-void FolderSelector::onHistoryPathSelected(wxEvent& event)
-{
-    //setFolderPathPhrase() => already called by onEditFolderPath() (wxEVT_COMMAND_COMBOBOX_SELECTED implies wxEVT_COMMAND_TEXT_UPDATED)
-
-    //notify action invoked by user
-    wxCommandEvent dummy(EVENT_ON_FOLDER_SELECTED);
-    ProcessEvent(dummy);
-}
-
-
 void FolderSelector::onEditFolderPath(wxCommandEvent& event)
 {
     setFolderPathPhrase(utfTo<Zstring>(event.GetString()), nullptr, folderComboBox_, staticText_);
 
-    wxCommandEvent dummy(EVENT_ON_FOLDER_MANUAL_EDIT);
+    wxCommandEvent dummy(EVENT_ON_FOLDER_SELECTED);
     ProcessEvent(dummy);
     event.Skip();
 }
@@ -199,7 +199,7 @@ void FolderSelector::onSelectFolder(wxCommandEvent& event)
     Zstring defaultFolderNative;
     {
         //make sure default folder exists: don't let folder picker hang on non-existing network share!
-        auto folderExistsTimed = [waitEndTime = std::chrono::steady_clock::now() + FOLDER_SELECTED_EXISTENCE_CHECK_TIME_MAX](const AbstractPath& folderPath)
+        auto folderAccessible = [stopTime = std::chrono::steady_clock::now() + FOLDER_SELECTED_EXISTENCE_CHECK_TIME_MAX](const AbstractPath& folderPath)
         {
             if (AFS::isNullPath(folderPath))
                 return false;
@@ -212,7 +212,7 @@ void FolderSelector::onSelectFolder(wxCommandEvent& event)
                 }
                 catch (FileError&) { return false; }
             });
-            return ft.wait_until(waitEndTime) == std::future_status::ready && ft.get(); //potentially slow network access: wait 200ms at most
+            return ft.wait_until(stopTime) == std::future_status::ready && ft.get(); //potentially slow network access: wait 200ms at most
         };
 
         auto trySetDefaultPath = [&](const Zstring& folderPathPhrase)
@@ -220,7 +220,7 @@ void FolderSelector::onSelectFolder(wxCommandEvent& event)
             if (acceptsItemPathPhraseNative(folderPathPhrase)) //noexcept
             {
                 const AbstractPath folderPath = createItemPathNative(folderPathPhrase);
-                if (folderExistsTimed(folderPath))
+                if (folderAccessible(folderPath))
                     if (const Zstring& nativePath = getNativeItemPath(folderPath);
                         !nativePath.empty())
                         defaultFolderNative = nativePath;
@@ -262,6 +262,8 @@ void FolderSelector::onSelectAltFolder(wxCommandEvent& event)
     Zstring folderPathPhrase = getPath();
     size_t parallelOps = getDeviceParallelOps_ ? getDeviceParallelOps_(folderPathPhrase) : 1;
 
+    const AbstractPath oldPath = createAbstractPath(folderPathPhrase);
+
     if (showCloudSetupDialog(parent_, folderPathPhrase, sftpKeyFileLastSelected_, parallelOps, static_cast<bool>(setDeviceParallelOps_)) != ConfirmationButton::accept)
         return;
 
@@ -271,8 +273,12 @@ void FolderSelector::onSelectAltFolder(wxCommandEvent& event)
         setDeviceParallelOps_(folderPathPhrase, parallelOps);
 
     //notify action invoked by user
-    wxCommandEvent dummy(EVENT_ON_FOLDER_SELECTED);
-    ProcessEvent(dummy);
+    if (createAbstractPath(folderPathPhrase) != oldPath)
+    {
+        wxCommandEvent dummy(EVENT_ON_FOLDER_SELECTED);
+        ProcessEvent(dummy);
+    }
+    //else: don't notify if user only changed connection settings, e.g. parallel Ops
 }
 
 
@@ -286,3 +292,13 @@ void FolderSelector::setPath(const Zstring& folderPathPhrase)
 {
     setFolderPathPhrase(folderPathPhrase, &folderComboBox_, folderComboBox_, staticText_);
 }
+
+
+void fff::openFolderInFileBrowser(const AbstractPath& folderPath) //throw FileError
+{
+        if (const Zstring& gdriveUrl = getGoogleDriveFolderUrl(folderPath); //throw FileError
+            !gdriveUrl.empty())
+            return openWithDefaultApp(gdriveUrl); //throw FileError
+        else
+            openWithDefaultApp(utfTo<Zstring>(AFS::getDisplayPath(folderPath))); //throw FileError
+}
